<?php
namespace Zeedhi\Framework\Process;

/**
 * Deals the process queue and uses private instances to prevent
 * modification, preventing php form copy-on-write and exhausting
 * php/apache/application memory.
 */
interface ProcessManager{
    /**
     * Adds a process to be executed on the queue.
     *
     * @param Process $process The callback to be executed.
     *
     * @return boolean TRUE if callback have been queued, FALSE otherwise.
     */
    public function addProcess($process);

    /**
     * Reads the state of a process.
     **
     * @param string $id The process id.
     *
     * @return string The process status.
     */
    public function getProcessStatus($id);

    /**
     * Starts a process from the queue before others by id.
     *
     * @param string $id The process id.
     *
     * @return boolean TRUE if callback could be called, FALSE otherwise.
     */
    public function run($id);

    /**
     * Removes a process from the queue before it's run without running it.
     *
     * @param string $id The process id.
     *
     * @return boolean TRUE if successfully threw it away, FALSE otherwise.
     */
    public function discard($id);

    /**
     * Kills all the processes being executed.
     */
    public function killAll();
}