import {
  Modal, ModalService, Button,
  IButtonEvent, Select, Grid,
  Checkbox, moment, IGridEvent,
  Text,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, IEventParam,
  Loader, Http, Config, Datasource,
} from '@zeedhi/core';
import { PmeMask } from '@/config';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pla00102RankingPrato {
    private modalRanking!: Modal;
    
    private utilities = Loader.getInstance('Utilities');
    
    public gridEditController = Loader.getInstance('GridEditableController');
    
    private appController = Loader.getInstance('AppController');
    
    public showReceive = false;
    
    public colsRanking = 12;
    
    private currentSelectedRegister: IDictionary = {}; // Atual Prato selecionado na tela ranking de pratos
    
    private currentSelectedRegisterCard: IDictionary = {}; // Atual coluna selecionada no cardapio
    
    private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
    
    public async createModal() {
      this.modalRanking = await ModalService.createFromJson('modals/pla00102_ranking_prato', true);
    }
    
    public cancelModal() {
      this.currentSelectedRegister = {};
      this.modalRanking.hide();
    }
    
    public async openModalRanking() {
      if (this.modalRanking) {
        this.currentSelectedRegister = {};
        this.modalRanking.show();
      } else {
        this.createModal().then(() => {
          this.modalRanking.show();
        });
      }
    }
    
    // Essa função realiza a mesma chamada da abertura da dados do prato.
    private insertNewCardia(cardapio: Cardapio) {
      const row = this.currentSelectedRegisterCard;
      Http.post('/getCardia/get', { requestType: 'Row', row }).then((response) => {
        const comensais = response.data.dataset.cardiaOnClickPlate[0]?.QTCOMESTCPME;
        const dtcardia = response.data.dataset.cardiaOnClickPlate[0]?.DTCARDIAPME;
        if (comensais && comensais !== 0) {
          cardapio.updateComens(comensais, dtcardia);
          this.currentSelectedRegisterCard.QTCOMESTCPME = comensais;
          this.openModalRanking();
        } else {
          this.utilities.showAlert('dinerNull', 'pme::comensalNulo', 'error');
        }
      });
    }
    
    public showModal({ event }: IEventParam<Button>) {
      (event as IDictionary).stopPropagation();
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      if (this.elaboracaoCardapio.getFilterData.NRCARDPME !== undefined) {
        this.onMountedCurrentRegister(cardapio);
        if (cardapio.dateLessDateOfProcess(this.currentSelectedRegisterCard.DTCARDIAPME)) { // Verifica se a data de processamento é maior q a data do cardapio.
          if (this.currentSelectedRegisterCard.CDTIPOPRAT !== undefined) { // Verifica se foi algum dados do comensais selecionado
            if (this.currentSelectedRegisterCard.NRCARDIAPME === null) this.insertNewCardia(cardapio);
            else this.openModalRanking();
          } else {
            this.utilities.showAlert('openRankingInComen', 'pme::comensaisSelecionado', 'error');
          }
        } else {
          this.utilities.showAlert('dateIsLessTheDateOfProcess', 'pme::foraDoPeriodoDeProcessamentoCard', 'error');
        }
      } else {
        this.utilities.showAlert('withoutFilter', 'pme::cardapioSemFiltro', 'error');
      }
    }
    
    private onMountedCurrentRegister(cardapio: Cardapio) {
      this.currentSelectedRegisterCard = cardapio.getCurrentSelectedRegister();
      const idcard = cardapio.dates.filter((data: IDictionary) => data.DTCARDIAPME === this.currentSelectedRegisterCard.DTCARDIAPME);
      
      this.currentSelectedRegisterCard.filterData = this.elaboracaoCardapio.getFilterData;
      this.currentSelectedRegisterCard.WEEKDAY = moment(this.currentSelectedRegisterCard.DTCARDIAPME, 'DD/MM/YYYY').day();
      this.currentSelectedRegisterCard.IDCARDESPEC = (idcard[0].IDCARDESPEC === undefined) ? 'N' : idcard[0].IDCARDESPEC;
      this.currentSelectedRegisterCard.QTCOMESTCPME = idcard[0].QTCOMESTCPME;
    }
    
    public fullscreen({ component }: IButtonEvent) {
      this.modalRanking.fullscreen = !this.modalRanking.fullscreen;
      component.iconName = (this.modalRanking.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
    }
    
    public filterPlates({ component }: IEventParam<Select>) {
      component.datasource.filter = {
        CDFILIAL: this.aggregatedFilter,
      };
      component.datasource.get();
    }
    
    public resetFinalPlate() {
      Metadata.getInstance<Select>('finalPlate').setValue(null);
    }
    
    public validPlates({ component }: IEventParam<Select>) {
      if (component.value) {
        const initialPlate = Metadata.getInstance<Select>('initialPlate');
        if (this.utilities.returnPrat(initialPlate.value, component.value)) {
          this.utilities.showAlert('Alert', 'pme::pratoIniMenor', 'error');
          component.setValue(null);
        } else {
          this.filterRankingPlate();
        }
      } else {
        this.filterRankingPlate();
      }
    }
    
    public filterRankingPlate() {
      const filter: IDictionary = [];
      this.mountedFilterRankingPlate(filter);
      
      const plateGrid = Metadata.getInstance<Grid>('plate-grid');
      plateGrid.datasource.filter = filter;
      plateGrid.datasource.get();
    }
    
    private changeVisibilityReceive(visible: boolean) {
      this.showReceive = visible;
    }
    
    private changeColsRanking() {
      this.colsRanking = (this.showReceive) ? 6 : 12;
    }
    
    private mountedFilterRankingPlate(filter: IDictionary) {
      const costZero = (Metadata.getInstance<Checkbox>('costZero').value) ? 'S' : 'N';
      this.utilities.onMountedFilterFind(filter, 'P_CDPRATO', '=', null);
      this.utilities.onMountedFilterFind(filter, 'P_CDFILIAL', '=', this.aggregatedFilter);
      this.utilities.onMountedFilterFind(filter, 'P_DTCARDAPIO', '=', this.currentSelectedRegisterCard.DTCARDIAPME);
      this.utilities.onMountedFilterFind(filter, 'P_TIPOCUSTO', '=', this.currentSelectedRegisterCard.filterData.planned);
      this.utilities.onMountedFilterFind(filter, 'P_CDPRATOFIN', '=', this.finalDateRankingFilter);
      this.utilities.onMountedFilterFind(filter, 'P_CDFILICUSTO', '=', this.baseCost);
      this.utilities.onMountedFilterFind(filter, 'P_CDSERVFILI', '=', this.currentSelectedRegisterCard.filterData.service.CDSERVFILI);
      this.utilities.onMountedFilterFind(filter, 'P_CDFILCENTPME', '=', this.currentSelectedRegisterCard.filterData.cdfilcentpme);
      this.utilities.onMountedFilterFind(filter, 'P_IMPCUSTOZERO', '=', costZero);
      this.utilities.onMountedFilterFind(filter, 'P_CDTIPOCARD', '=', this.currentSelectedRegisterCard.filterData.typeMenu.CDTIPOCARD);
      this.utilities.onMountedFilterFind(filter, 'P_MAXNIVEL', '=', (Config.masks.plateMask as PmeMask).nivelMax);
      this.utilities.onMountedFilterFind(filter, 'P_CDPRATOINI', '=', Metadata.getInstance<Select>('initialPlate').value);
    }
    
    public onMountedGridRankingDePratos() {
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      const registroAtual = cardapio.getCurrentSelectedRegister();
      if (registroAtual) {
        Metadata.getInstance<Text>('valorTipoDePrato').text = registroAtual.NMTIPOPRAT;
        Metadata.getInstance<Text>('valorData').text = moment(registroAtual.DTCARDIAPME.substring(0, 10), 'DD/MM/YYYY').format(this.utilities.getFormatLocale());
      }
    }
    
    public onChangeCurrentRowFilterReceive({ component }: IEventParam<Datasource>) {
      const plateGrid = Metadata.getInstance<Grid>('plate-grid');
      const filter: IDictionary = [];
      if (component.currentRow !== null) {
        if (Object.keys(component.currentRow).length > 0) {
          this.currentSelectedRegister = component.currentRow;
          this.mountedFilterRankingReceive(filter, component.currentRow);
          const receiveGrid = Metadata.getInstance<Grid>('receive-grid');
          if (component.currentRow.IDATIVO !== 'N') {
            this.changeVisibilityReceive(plateGrid.datasource.data.length > 0);
            this.changeColsRanking();
            receiveGrid.isVisible = true;
            receiveGrid.datasource.filter = filter;
          } else {
            this.utilities.showAlert('inativo', 'pme::pratoReceitaInativa', 'warning');
            receiveGrid.datasource.data = [];
            receiveGrid.datasource.filter = {};
            this.gridEditController.reload({ receiveGrid });
          }
        }
      }
    }
    
    private mountedFilterRankingReceive(filter: IDictionary, selectedPlate: IDictionary) {
      this.utilities.onMountedFilterFind(filter, 'P_CDFILIAL', '=', this.aggregatedFilter);
      this.utilities.onMountedFilterFind(filter, 'P_CDPRATO', '=', selectedPlate.CDPRATO);
      this.utilities.onMountedFilterFind(filter, 'P_CDFILAUX', '=', selectedPlate.CDFILAUXPRAT);
      this.utilities.onMountedFilterFind(filter, 'P_NRTIPOCUSTO', '=', this.currentSelectedRegisterCard.filterData.planned);
      this.utilities.onMountedFilterFind(filter, 'P_DTCUSTO', '=', this.dtCusto);
      this.utilities.onMountedFilterFind(filter, 'P_CDFILICUSTO', '=', this.baseCost);
      this.utilities.onMountedFilterFind(filter, 'P_CDSERVFILI', '=', this.currentSelectedRegisterCard.filterData.service.CDSERVFILI);
      this.utilities.onMountedFilterFind(filter, 'P_CDFILCENTPME', '=', this.currentSelectedRegisterCard.filterData.cdfilcentpme);
      this.utilities.onMountedFilterFind(filter, 'P_CDTIPOCARD', '=', this.currentSelectedRegisterCard.filterData.typeMenu.CDTIPOCARD);
    }
    
    public saveCurrentSelectedRegister({ component }: IEventParam<Button>) {
      if (Object.keys(this.getCurrentSelectedRegister).length > 0) {
        const row: IDictionary = {};
        this.mountSaveData(row);
        this.utilities.activeLoadingButton(component);
        if (this.validReceita(row)) {
          if (row.selectedPlate.IDATIVO !== 'N') {
            Http.post('plate/new', { requestType: 'Row', row }).then((response) => {
              if (response.data.dataset.plateSaved[0]) {
                this.updateCurrentSelectedRegisterCard(row);
                this.cancelModal();
                this.utilities.showAlert('plateSaved', 'pme::salvoComSucesso', 'success');
              }
            }).finally(() => {
              this.utilities.disableLoadingButton(component);
            });
          } else {
            this.utilities.showAlert('saveWithoutPlate', 'pme::pratoInativoSelecionado', 'error');
            component.disabled = false;
          }
        } else {
          component.disabled = false;
        }
      } else this.utilities.showAlert('saveWithoutPlate', 'pme::pratoNaoSelecionado', 'error');
    }
    
    private validReceita(row: IDictionary): boolean {
      if (row.selectedPlate.IDRECEITA === 'N') {
        this.utilities.showAlert('saveWithoutPlate', 'pme::pratoSemReceita', 'error');
        return false;
      }
      
      if (row.selectedPlate.IDRECEITA === 'P') {
        if (row.selectedPlate.IDUTRPADCA === 'S') return true;
        
        this.utilities.showAlert('saveWithoutPlate', 'pme::pratoSemReceitaRevisada', 'error');
        return false;
      }
      
      return true;
    }
    
    private updateCurrentSelectedRegisterCard(row: IDictionary) {
      this.currentSelectedRegisterCard.CDPRATO = this.currentSelectedRegister.CDPRATO;
      this.currentSelectedRegisterCard.NMPRATO = this.currentSelectedRegister.NMPRATO;
      this.currentSelectedRegisterCard.CDFILAUXPRAT = this.currentSelectedRegister.CDFILAUXPRAT;
      this.currentSelectedRegisterCard.VRPECONSCPME = (this.currentSelectedRegisterCard.VRPECONSCPME === null) ? 100 : row.numberDiners;
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      cardapio.forceUpdateInCard();
    }
    
    private mountSaveData(row: IDictionary) {
      row.selectedPlate = this.getCurrentSelectedRegister;
      row.plateData = [[this.currentSelectedRegisterCard]];
      row.numberDiners = (this.currentSelectedRegisterCard.VRPECONSCPME === null) ? this.currentSelectedRegisterCard.QTCOMESTCPME : parseInt(this.currentSelectedRegisterCard.VRPECONSCPME, 10);
    }
    
    private get baseCost() {
      if (this.currentSelectedRegisterCard.filterData.aggregated !== null) {
        if (!this.currentSelectedRegisterCard.filterData.baseCost) return this.currentSelectedRegisterCard.filterData.aggregated.CDFILIAL;
      }
      return this.currentSelectedRegisterCard.filterData.cdfilcentpme;
    }
    
    private get finalDateRankingFilter() {
      let finalDate = Metadata.getInstance<Select>('finalPlate').value;
      if (finalDate !== null) finalDate += 'z';
      return finalDate;
    }
    
    private get aggregatedFilter() {
      if (this.currentSelectedRegisterCard.filterData.aggregated !== null) {
        return this.currentSelectedRegisterCard.filterData.aggregated.CDFILIAL;
      }
      return this.currentSelectedRegisterCard.filterData.cdfilcentpme;
    }
    
    private get dtCusto() {
      if (this.currentSelectedRegisterCard.filterData.radioSelect === 'C' || this.currentSelectedRegisterCard.filterData.radioSelect === undefined) {
        return this.currentSelectedRegisterCard.DTCARDIAPME;
      }
      return moment(this.currentSelectedRegisterCard.filterData.referenceDate).format('DD/MM/YYYY');
    }
    
    private get getCurrentSelectedRegister() {
      return this.currentSelectedRegister;
    }
    
    public modifyColor({ row }: IGridEvent) {
      if (row.IDATIVO === 'N') return 'inactive-plate';
      return '';
    }
    
    public pratoComponentProps = {
      component: 'ZdSelect',
      dataText: [
        {
          name: 'CDPRATO',
          mask: (Config.masks.plateMask as PmeMask).pattern,
        },
        'NMPRATO',
      ],
      formatterDataText: [
        {
          name: 'CDPRATO',
          mask: (Config.masks.plateMask as PmeMask).pattern,
        },
        'NMPRATO',
      ],
      dataValueOut: [
        {
          column: 'CDPRATO',
          columnOnGrid: 'CDPRATO',
        },
        {
          column: 'NMPRATO',
          columnOnGrid: 'NMPRATO',
        },
      ],
      dataValue: 'CDPRATO',
    };
    
    public produtoComponentProps = {
      component: 'ZdSelect',
      dataText: [
        {
          name: 'CDARVPROD',
          mask: (Config.masks.productMask as PmeMask).pattern,
        },
        'NMPRODUTO',
      ],
      formatterDataText: [
        {
          name: 'CDARVPROD',
          mask: (Config.masks.productMask as PmeMask).pattern,
        },
        'NMPRODUTO',
      ],
      dataValueOut: [
        {
          column: 'CDARVPROD',
          columnOnGrid: 'CDARVPROD',
        },
        {
          column: 'NMPRODUTO',
          columnOnGrid: 'NMPRODUTO',
        },
      ],
      dataValue: 'CDPRODUTO',
    };

    public closeReceita() {
      this.showReceive = false;
      this.changeColsRanking();
    }
}
