import {
  Loader, Metadata, IEventParam, IDictionary, Config, I18n,
} from '@zeedhi/core';
import {
  Form, Select, Button, CheckboxMultiple,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme01100CustoPlanejadoRealizadoController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');
  
  public modalController = Loader.getInstance('ModalController');

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }
  
  private impTipoProduto = {
    ESTOCAVEIS: 'S',
    NAOESTOCAVEIS: 'S',
  };
  
  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const nrorg = localStorage.getItem('NRORG');
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
      {
        name: 'NRORG',
        operator: '=',
        value: nrorg,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterAgregAgrup({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const agrupUnidValue = agrupUnid.value ? agrupUnid.value.CDGRUPFILI : 'T';
    
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  private setParams(row: IDictionary) {
    const mask: string = (Config.masks.productMask as PmeMask).pattern;
    row.CDSERVFILI = row.SERVICO ? row.SERVICO : null;
    row.CDFILIAL = row.UNIDADE ? row.UNIDADE : null;
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.CDGRUPFILI = row.GRUPFILI ? row.GRUPFILI.CDGRUPFILI : null;
    row.NMGRUPFILI = row.GRUPFILI ? row.GRUPFILI.NMGRUPFILI : I18n.translate('pme::nenhum');
    row.TAMACNIVEL = this.utilities.calcTamacNivel(row.NIVEL, 'produto');
    row.ESTOCAVEIS = this.impTipoProduto.ESTOCAVEIS;
    row.NAOESTOCAVEIS = this.impTipoProduto.NAOESTOCAVEIS;
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    row.AGRUPMASK = this.utilities.getReportMaskNivel(mask, row.NIVEL);
    row.DTMESINIC = row.DTDE;
    row.DTMESFINA = row.DTATE;
    row.CDTIPOCUSTOREAL = row.TIPOCUSTOREAL.CDTIPOCUSTO;
    row.NMTIPOCUSTOREAL = row.TIPOCUSTOREAL.NMTIPOCUSTO;
    row.CDTIPOCUSTOPLAN = row.TIPOCUSTOPLAN.CDTIPOCUSTO;
    row.NMTIPOCUSTOPLAN = row.TIPOCUSTOPLAN.NMTIPOCUSTO;
  }
  
  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');
    const tipoProduto = Metadata.getInstance<CheckboxMultiple>('TIPOPRODUTO');

    if (this.reportController.formIsValid(form)) {
      const row: IDictionary = form.value;
      
      this.setParams(row);
      if (row.CDFILCENTPME == null) this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'error');
      if (tipoProduto.value.length === 0) this.utilities.showAlert('alertError', 'pme::tipoProdutoDeveInformar', 'error');
      else {
        row.reportFormat = component.name;
        this.utilities.activeLoadingButton(component);
        this.reportController.generateReport('report/custoPlanRealCard', row, row.CD_REPORT_CODE).finally(() => {
          this.utilities.disableLoadingButton(component);
        });
      }
    }
  }
  
  public setTipoProduto({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = component.value.join('_');
      if (items.includes('ESTOCAVEIS')) this.impTipoProduto.ESTOCAVEIS = 'S';
      else this.impTipoProduto.ESTOCAVEIS = 'N';

      if (items.includes('NAOESTOC')) this.impTipoProduto.NAOESTOCAVEIS = 'S';
      else this.impTipoProduto.NAOESTOCAVEIS = 'N';
    }
  }
  
  public setFilterServAgrupUnid({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const unidadeValue = (unidade.value.length !== 0) ? unidade.value.join('_') : 'T';
    
    const grupserv = Metadata.getInstance<Select>('GRUPOSERVICO');
    const grupServValue = grupserv.value ? grupserv.value : 'T';
    
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const agrupUnidValue = agrupUnid.value ? agrupUnid.value.CDGRUPFILI : 'T';

    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidadeValue,
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPSERV',
        operator: '=',
        value: grupServValue,
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
}
